"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var indexes_exports = {};
__export(indexes_exports, {
  extractSecondaryIndexes: () => extractSecondaryIndexes
});
module.exports = __toCommonJS(indexes_exports);
var import_assert = __toESM(require("assert"));
var import_helpers = require("../helpers");
var import_primary_key = require("./primary-key");
function extractSecondaryIndexes(config, type, fieldMap) {
  const gsis = extractGSIs(config, type, fieldMap);
  const lsis = extractLSIs(config, type, fieldMap);
  const simpleIndexes = extractSimpleIndexes(type, fieldMap);
  const indexes = [...gsis, ...lsis, ...simpleIndexes];
  if ((0, import_helpers.hasInterface)("PublicModel", type)) {
    const publicIdIndex = {
      isComposite: false,
      isSingleField: true,
      name: "publicId",
      partitionKeyFields: [(0, import_primary_key.getFieldFromFieldMap)(fieldMap, "publicId")],
      partitionKeyName: "publicId",
      projectionType: (0, import_helpers.hasDirective)("public", type) ? getProjectionType((0, import_helpers.getDirective)("public", type)) : "all",
      type: "gsi"
    };
    indexes.push(publicIdIndex);
  }
  return indexes;
}
function extractGSIs(config, type, fieldMap) {
  if (!type.astNode?.directives) {
    return [];
  }
  return type.astNode.directives.filter((directive) => directive.name.value === "gsi").map((directive) => {
    const sortKeyFields = (0, import_helpers.getOptionalArgStringArrayValue)(
      "skFields",
      directive
    )?.map((fieldName) => (0, import_primary_key.getFieldFromFieldMap)(fieldMap, fieldName));
    const sortKeyPrefix = (0, import_helpers.getOptionalArgStringValue)("skPrefix", directive);
    const name = (0, import_helpers.getArgStringValue)("name", directive);
    const pk = pkInfo(config, type, fieldMap, directive);
    if (sortKeyFields || sortKeyPrefix) {
      const sk = skInfo(config, type, fieldMap, directive);
      return {
        isComposite: true,
        name,
        partitionKeyFields: pk.fields,
        partitionKeyIsSingleField: pk.isSingleField,
        partitionKeyName: pk.name,
        partitionKeyPrefix: pk.prefix,
        projectionType: getProjectionType(directive),
        sortKeyFields: sk.fields,
        sortKeyIsSingleField: sk.isSingleField,
        sortKeyName: sk.name,
        sortKeyPrefix: sk.prefix,
        type: "gsi"
      };
    }
    return {
      isComposite: false,
      isSingleField: pk.isSingleField,
      name,
      partitionKeyFields: pk.fields,
      partitionKeyName: pk.name,
      partitionKeyPrefix: pk.prefix,
      projectionType: getProjectionType(directive),
      type: "gsi"
    };
  });
}
function extractLSIs(config, type, fieldMap) {
  if (!type.astNode?.directives) {
    return [];
  }
  return type.astNode.directives.filter((directive) => directive.name.value === "lsi").map((directive) => {
    const sortKeyFields = (0, import_helpers.getArgStringArrayValue)("skFields", directive).map(
      (fieldName) => (0, import_primary_key.getFieldFromFieldMap)(fieldMap, fieldName)
    );
    (0, import_assert.default)(
      sortKeyFields.length > 0,
      "LSI must have at least one sort key field"
    );
    const primaryKey = (0, import_primary_key.extractPrimaryKey)(type, fieldMap);
    const name = (0, import_helpers.getArgStringValue)("name", directive);
    const sk = skInfo(config, type, fieldMap, directive);
    return {
      isComposite: true,
      name,
      partitionKeyFields: primaryKey.partitionKeyFields,
      partitionKeyIsSingleField: primaryKey.isComposite ? primaryKey.partitionKeyIsSingleField : primaryKey.isSingleField,
      partitionKeyName: primaryKey.partitionKeyName,
      partitionKeyPrefix: primaryKey.partitionKeyPrefix,
      projectionType: getProjectionType(directive),
      sortKeyFields: sk.fields,
      sortKeyIsSingleField: sk.isSingleField,
      sortKeyName: sk.name,
      sortKeyPrefix: sk.prefix,
      type: "lsi"
    };
  });
}
function extractSimpleIndexes(type, fieldMap) {
  if (!type.astNode?.fields) {
    return [];
  }
  return type.astNode.fields.filter(
    (field) => field.directives?.map(({ name }) => name.value).includes("simpleIndex")
  ).map((fieldNode) => {
    const field = fieldMap[fieldNode.name.value];
    const directive = (0, import_helpers.getDirective)("simpleIndex", fieldNode);
    return {
      isComposite: false,
      isSingleField: true,
      name: field.fieldName,
      partitionKeyFields: [field],
      partitionKeyName: field.fieldName,
      projectionType: getProjectionType(directive),
      type: "gsi"
    };
  });
}
function getProjectionType(directive) {
  const arg = (0, import_helpers.getOptionalArg)("projection", directive);
  if (!arg) {
    return "all";
  }
  (0, import_assert.default)(
    arg.value.kind === "EnumValue",
    `Expected projection to be an enum value`
  );
  const type = arg.value.value.toLowerCase();
  (0, import_assert.default)(
    type === "all" || type === "keys_only",
    `Invalid projection type ${type}`
  );
  return type;
}
function pkInfo(config, type, fieldMap, directive) {
  const indexName = (0, import_helpers.getArgStringValue)("name", directive);
  const prefix = (0, import_helpers.getOptionalArgStringValue)("pkPrefix", directive);
  const fields = (0, import_helpers.getArgStringArrayValue)("pkFields", directive).map(
    (fieldName) => (0, import_primary_key.getFieldFromFieldMap)(fieldMap, fieldName)
  );
  (0, import_assert.default)(fields.length > 0, "GSI must have at least one partition key field");
  const isSingleField = fields.length === 1;
  const canUseEfficientIndex = config.useEfficientIndexes && isSingleField && !prefix;
  const name = canUseEfficientIndex ? fields[0].fieldName : `${indexName}pk`;
  return {
    fields,
    isSingleField,
    name,
    prefix
  };
}
function skInfo(config, type, fieldMap, directive) {
  const indexName = (0, import_helpers.getArgStringValue)("name", directive);
  const prefix = (0, import_helpers.getOptionalArgStringValue)("skPrefix", directive);
  const fields = (0, import_helpers.getOptionalArgStringArrayValue)("skFields", directive)?.map(
    (fieldName) => (0, import_primary_key.getFieldFromFieldMap)(fieldMap, fieldName)
  );
  if (!fields?.length) {
    (0, import_assert.default)(
      prefix,
      "Composite GSI must have at least one sort key field or prefix"
    );
    return {
      fields: [],
      isSingleField: false,
      name: `${indexName}sk`,
      prefix
    };
  }
  const isSingleField = fields.length === 1;
  const canUseEfficientIndex = config.useEfficientIndexes && isSingleField && !prefix;
  const name = canUseEfficientIndex ? fields[0].fieldName : `${indexName}sk`;
  return {
    fields,
    isSingleField,
    name,
    prefix
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  extractSecondaryIndexes
});
