"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var tables_exports = {};
__export(tables_exports, {
  assertPrimaryKeysMatch: () => assertPrimaryKeysMatch,
  extractTable: () => extractTable,
  extractTableName: () => extractTableName
});
module.exports = __toCommonJS(tables_exports);
var import_assert = require("@code-like-a-carpenter/assert");
var import_cdc = require("./extractors/cdc");
var import_helpers = require("./helpers");
var import_models = require("./models");
var import_paths = require("./paths");
const schemaTables = /* @__PURE__ */ new WeakMap();
function extractTable(config, schema, type, outputFile) {
  let tables = schemaTables.get(schema);
  if (!tables) {
    tables = /* @__PURE__ */ new Map();
    schemaTables.set(schema, tables);
  }
  const tableName = extractTableName(type);
  const existingTable = tables.get(tableName);
  if (existingTable) {
    return updateExistingTable(
      config,
      type,
      existingTable,
      tableName,
      outputFile
    );
  }
  const newTable = defineNewTable(config, type, tableName, outputFile);
  tables.set(newTable.tableName, newTable);
  return newTable;
}
function extractTableName(type) {
  const directive = (0, import_helpers.getOptionalDirective)("table", type);
  if (directive) {
    const value = (0, import_helpers.getOptionalArgStringValue)("name", directive);
    if (value) {
      return value;
    }
  }
  return `Table${type.name}`;
}
function assertPrimaryKeysMatch(tableName, tablePrimaryKey, modelPrimaryKey) {
  (0, import_assert.assert)(
    tablePrimaryKey.isComposite === modelPrimaryKey.isComposite,
    `All models in table must be composite (or not). Please check the definitions for the models in ${tableName}.`
  );
}
function combineSecondaryIndexes(tableName, tableIndexes, modelIndexes) {
  const tableIndexMap = new Map(
    tableIndexes.map((t) => [t.name, t])
  );
  const modelIndexMap = new Map(
    modelIndexes.map((m) => [m.name, m])
  );
  for (const [name, modelIndex] of modelIndexMap.entries()) {
    const index = tableIndexMap.get(name);
    if (index) {
      if (index.type === "gsi" && modelIndex.type === "gsi") {
        (0, import_assert.assert)(
          index.isComposite === modelIndex.isComposite,
          `Please check the secondary index ${name} for the table ${tableName}. All indexes of the same name must be of the same type (either partition or composite).`
        );
      }
      (0, import_assert.assert)(
        index.type === modelIndex.type,
        `Please check the secondary index ${name} for the table ${tableName}. All indexes of the same name must be of the same type (either gsi or lsi).`
      );
      (0, import_assert.assert)(
        index.projectionType === modelIndex.projectionType,
        `Please check the secondary index ${name} for the table ${tableName}. All indexes of the same name must be of the same projection type (either "all"" or "keys_only").`
      );
    } else {
      tableIndexes.push(modelIndexToTableIndex(modelIndex));
    }
  }
}
function defineNewTable(config, type, tableName, outputFile) {
  const model = (0, import_models.getModel)(type);
  const dependenciesModuleId = (0, import_paths.resolveDependenciesModuleId)(config, outputFile);
  const table = {
    dependenciesModuleId,
    enablePointInTimeRecovery: shouldEnablePointInTimeRecovery(config, type),
    enableStreaming: shouldEnableStreaming(type, model),
    hasPublicModels: (0, import_helpers.hasInterface)("PublicModel", type),
    hasTtl: !!model.ttlConfig,
    primaryKey: model.primaryKey,
    secondaryIndexes: model.secondaryIndexes.map(modelIndexToTableIndex),
    tableName
  };
  if (model.changeDataCaptureConfig.length) {
    return {
      ...table,
      dispatcherConfig: (0, import_cdc.extractDispatcherConfig)(config, outputFile, type),
      hasCdc: true
    };
  }
  return { ...table, hasCdc: false };
}
function modelIndexToTableIndex(mi) {
  if (mi.type === "lsi") {
    return {
      isComposite: mi.isComposite,
      name: mi.name,
      partitionKeyIsSingleField: mi.partitionKeyIsSingleField,
      partitionKeyName: mi.partitionKeyName,
      projectionType: mi.projectionType,
      sortKeyIsSingleField: mi.sortKeyIsSingleField,
      sortKeyName: mi.sortKeyName,
      type: "lsi"
    };
  }
  if (mi.isComposite) {
    return {
      isComposite: mi.isComposite,
      name: mi.name,
      partitionKeyIsSingleField: mi.partitionKeyFields.length === 1,
      partitionKeyName: mi.partitionKeyName,
      projectionType: mi.projectionType,
      sortKeyIsSingleField: mi.isComposite && mi.sortKeyFields.length === 1,
      sortKeyName: mi.sortKeyName,
      type: "gsi"
    };
  }
  return {
    isComposite: mi.isComposite,
    name: mi.name,
    partitionKeyIsSingleField: mi.partitionKeyFields.length === 1,
    partitionKeyName: mi.partitionKeyName,
    partitionKeyPrefix: mi.partitionKeyPrefix,
    projectionType: mi.projectionType,
    type: "gsi"
  };
}
function updateExistingTable(config, type, existingTable, tableName, outputFile) {
  const model = (0, import_models.getModel)(type);
  existingTable.enablePointInTimeRecovery = existingTable.enablePointInTimeRecovery || shouldEnablePointInTimeRecovery(config, type);
  existingTable.enableStreaming = existingTable.enableStreaming || shouldEnableStreaming(type, model);
  existingTable.hasPublicModels = existingTable.hasPublicModels || (0, import_helpers.hasInterface)("PublicModel", type);
  if (existingTable.hasCdc || model.changeDataCaptureConfig.length > 0) {
    existingTable.hasCdc = true;
    existingTable.dispatcherConfig = (0, import_cdc.extractDispatcherConfig)(
      config,
      outputFile,
      type
    );
  }
  existingTable.hasTtl = existingTable.hasTtl || !!model.ttlConfig;
  assertPrimaryKeysMatch(tableName, existingTable.primaryKey, model.primaryKey);
  combineSecondaryIndexes(
    tableName,
    existingTable.secondaryIndexes,
    model.secondaryIndexes
  );
  return existingTable;
}
function shouldEnablePointInTimeRecovery(config, type) {
  const tableDirective = (0, import_helpers.getOptionalDirective)("table", type);
  if (!tableDirective) {
    return config.tableDefaults.enablePointInTimeRecovery;
  }
  return (0, import_helpers.getOptionalArgBooleanValue)("enablePointInTimeRecovery", tableDirective) !== false;
}
function shouldEnableStreaming(type, model) {
  if (model.changeDataCaptureConfig.length > 0) {
    return true;
  }
  const tableDirective = (0, import_helpers.getOptionalDirective)("table", type);
  if (!tableDirective) {
    return false;
  }
  return (0, import_helpers.getOptionalArgBooleanValue)("enableStreaming", tableDirective) === true;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  assertPrimaryKeysMatch,
  extractTable,
  extractTableName
});
