import assert from "assert";
import path from "path";
import { assertObjectType, isObjectType } from "graphql";
import { filterNull, hasInterface } from "./helpers";
import { extractModel } from "./models";
import { extractTable } from "./tables";
function parse(schema, documents, config, info) {
  const outputFile = info?.outputFile;
  assert(outputFile, "outputFile is required");
  const dependenciesModuleId = resolveDependenciesModuleId(config, {
    outputFile
  });
  const typesMap = schema.getTypeMap();
  const models = Array.from(
    new Set(
      Object.keys(typesMap).map((typeName) => schema.getTypeMap()[typeName]).filter((type) => isObjectType(type)).map((type) => assertObjectType(type)).filter((type) => hasInterface("Model", type)).map((type) => extractModel(config, schema, type, outputFile)).sort()
    )
  );
  const tables = Array.from(
    new Set(
      Object.keys(typesMap).map((typeName) => schema.getTypeMap()[typeName]).filter((type) => isObjectType(type)).map((type) => assertObjectType(type)).filter((type) => hasInterface("Model", type)).map((type) => extractTable(config, schema, type, outputFile)).sort()
    )
  );
  return {
    additionalImports: models.flatMap((model) => model.fields.map((field) => field.computeFunction)).filter(filterNull),
    dependenciesModuleId,
    models,
    tables
  };
}
function resolveDependenciesModuleId(config, info) {
  assert(
    info?.outputFile,
    "You appear to be using this plugin in a context that does not require an output file. This is not supported."
  );
  const { dependenciesModuleId } = config;
  const { outputFile } = info;
  return dependenciesModuleId.startsWith(".") ? path.relative(
    path.resolve(process.cwd(), path.dirname(outputFile)),
    path.resolve(process.cwd(), dependenciesModuleId)
  ) : dependenciesModuleId;
}
export {
  parse
};
